unit Input;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Grids;

const
  MaxState = 8;

type
  RuleSpecification = array[1..MaxState, 1..MaxState] of Integer;
  TestOperators = (OR_GE, OR_EQ, AND_GE, AND_EQ);

  TInputDialog = class(TForm)
    BirthControl: TGroupBox;
    BirthGrid: TStringGrid;
    BirthColumnLabel: TLabel;
    BirthRowLabel1: TLabel;
    BirthRowLabel2: TLabel;
    BirthRowLabel3: TLabel;
    BirthRowLabel4: TLabel;
    BirthRowLabel5: TLabel;
    BirthRowLabel6: TLabel;
    BirthRowLabel7: TLabel;
    BirthRowLabel8: TLabel;
    BirthOperatorSelector: TButton;
    BirthResetButton: TButton;
    BirthRandomButton: TButton;
    BirthRandomLimitEdit: TLabel;
    SurvivalControl: TGroupBox;
    SurvivalGrid: TStringGrid;
    SurvivalColumnLabel: TLabel;
    SurvivalRowLabel1: TLabel;
    SurvivalRowLabel2: TLabel;
    SurvivalRowLabel3: TLabel;
    SurvivalRowLabel4: TLabel;
    SurvivalRowLabel5: TLabel;
    SurvivalRowLabel6: TLabel;
    SurvivalRowLabel7: TLabel;
    SurvivalRowLabel8: TLabel;
    SurvivalOperatorSelector: TButton;
    SurvivalResetButton: TButton;
    SurvivalRandomButton: TButton;
    SurvivalRandomLimitEdit: TLabel;
    RuleDisplay: TEdit;
    OKButton: TButton;
    CancelButton: TButton;
    ByLine: TLabel;
    procedure FormActivate(Sender: TObject);
    procedure InitOperatorSelectors;
    procedure InitRuleGrids;
    procedure BirthGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure SurvivalGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure BirthGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure SurvivalGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BirthOperatorSelectorClick(Sender: TObject);
    procedure BirthResetButtonClick(Sender: TObject);
    procedure BirthRandomLimitEditClick(Sender: TObject);
    procedure BirthRandomButtonClick(Sender: TObject);
    procedure SurvivalOperatorSelectorClick(Sender: TObject);
    procedure SurvivalResetButtonClick(Sender: TObject);
    procedure SurvivalRandomLimitEditClick(Sender: TObject);
    procedure SurvivalRandomButtonClick(Sender: TObject);

    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
  public
  end;

var
  RuleString: String;
  BirthRandomLimit: Integer;
  SurvivalRandomLimit: Integer;
  Birth: RuleSpecification;
  Survival: RuleSpecification;
  BirthTestOperators: TestOperators;
  SurvivalTestOperators: TestOperators;
  DecCells: Boolean;

procedure SaveRuleString(S: String);
procedure InitBirths;
procedure InitSurvivals;
procedure InitVars;
procedure InitRuleFromString(S: String);
function GetRuleAsString: String;

implementation

{$R *.DFM}

procedure TInputDialog.FormActivate(Sender: TObject);
begin
  InitRuleFromString(RuleString);
  InitOperatorSelectors;
  BirthRandomLimit := StrToInt(BirthRandomLimitEdit.Caption) + 1;
  SurvivalRandomLimit := StrToInt(SurvivalRandomLimitEdit.Caption) + 1;
  InitRuleGrids
end;

procedure TInputDialog.InitOperatorSelectors;
begin
  case BirthTestOperators of
    OR_GE: BirthOperatorSelector.Caption := 'OR     >=';
    OR_EQ: BirthOperatorSelector.Caption := 'OR      =';
    AND_GE: BirthOperatorSelector.Caption := 'AND   >=';
    AND_EQ: BirthOperatorSelector.Caption := 'AND    ='
  end;
  case SurvivalTestOperators of
    OR_GE: SurvivalOperatorSelector.Caption := 'OR     >=';
    OR_EQ: SurvivalOperatorSelector.Caption := 'OR      =';
    AND_GE: SurvivalOperatorSelector.Caption := 'AND   >=';
    AND_EQ: SurvivalOperatorSelector.Caption := 'AND    ='
  end
end;

procedure TInputDialog.InitRuleGrids;
var
  I, J: Integer;
begin
  with BirthGrid do
    for I := 0 to ColCount - 1 do
      for J := 0 to RowCount - 1 do
        Cells[I, J] := IntToStr(Birth[J + 1, I + 1]);
  with SurvivalGrid do
    for I := 0 to ColCount - 1 do
      for J := 0 to RowCount - 1 do
        SurvivalGrid.Cells[I, J] := IntToStr(Survival[J + 1, I + 1]);
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInputDialog.BirthGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  with BirthGrid do
  begin
    if DecCells = true then
      if Cells[ACol, ARow] = '0' then
        Cells[ACol, ARow] := '6'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) - 1)
    else
      if Cells[ACol, ARow] = '6' then
        Cells[ACol, ARow] := '0'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) + 1);
    Birth[ARow + 1, ACol + 1] := StrToInt(Cells[ACol, ARow])
  end;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInputDialog.SurvivalGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  with SurvivalGrid do
  begin
    if DecCells = true then
      if Cells[ACol, ARow] = '0' then
        Cells[ACol, ARow] := '6'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) - 1)
    else
      if Cells[ACol, ARow] = '6' then
        Cells[ACol, ARow] := '0'
      else
        Cells[ACol, ARow] := IntToStr(StrToInt(Cells[ACol, ARow]) + 1);
    Survival[ARow + 1, ACol + 1] := StrToInt(Cells[ACol, ARow])
  end;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInputDialog.BirthGridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TStringGrid) do
  begin
    Canvas.TextRect(Rect, Rect.Left + 6, Rect.Top + 3, cells[ACol, ARow])
  end
end;

procedure TInputDialog.SurvivalGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with (Sender as TStringGrid) do
  begin
    Canvas.TextRect(Rect, Rect.Left + 6, Rect.Top + 3, cells[ACol, ARow])
  end
end;

procedure TInputDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ssShift in Shift then
    DecCells := true
end;

procedure TInputDialog.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  DecCells := false
end;

procedure TInputDialog.BirthOperatorSelectorClick(Sender: TObject);
begin
  if BirthTestOperators = AND_EQ then
    BirthTestOperators := OR_GE
  else
    BirthTestOperators := Succ(BirthTestOperators);
  InitOperatorSelectors;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInputDialog.BirthRandomLimitEditClick(Sender: TObject);
begin
  with BirthRandomLimitEdit do
  begin
    if DecCells = true then
      if Caption = '1' then
        Caption := '6'
      else
        Caption := IntToStr(StrToInt(Caption) - 1)
    else
      if Caption = '6' then
        Caption := '1'
      else
        Caption := IntToStr(StrToInt(Caption) + 1);
    BirthRandomLimit := StrToInt(Caption) + 1
  end
end;

procedure TInputDialog.BirthResetButtonClick(Sender: TObject);
begin
  InitBirths;
  InitRuleGrids
end;

procedure TInputDialog.BirthRandomButtonClick(Sender: TObject);
var
  I: Integer;
begin
  Randomize;
  for I := 1 to MaxState do
  begin
    Birth[I, Random(MaxState) + 1] := Random(BirthRandomLimit);
    Birth[I, Random(MaxState) + 1] := Random(BirthRandomLimit)
  end;
  InitRuleGrids
end;

procedure TInputDialog.SurvivalOperatorSelectorClick(Sender: TObject);
begin
  if SurvivalTestOperators = AND_EQ then
    SurvivalTestOperators := OR_GE
  else
    SurvivalTestOperators := Succ(SurvivalTestOperators);
  InitOperatorSelectors;
  RuleDisplay.Text := GetRuleAsString
end;

procedure TInputDialog.SurvivalRandomLimitEditClick(Sender: TObject);
begin
  with SurvivalRandomLimitEdit do
  begin
    if DecCells = true then
      if Caption = '1' then
        Caption := '6'
      else
        Caption := IntToStr(StrToInt(Caption) - 1)
    else
      if Caption = '6' then
        Caption := '1'
      else
        Caption := IntToStr(StrToInt(Caption) + 1);
    SurvivalRandomLimit := StrToInt(Caption) + 1
  end
end;

procedure TInputDialog.SurvivalResetButtonClick(Sender: TObject);
begin
  InitSurvivals;
  InitRuleGrids
end;

procedure TInputDialog.SurvivalRandomButtonClick(Sender: TObject);
var
  I: Integer;
begin
  Randomize;
  for I := 1 to MaxState do
  begin
    Survival[I, Random(MaxState) + 1] := Random(SurvivalRandomLimit);
    Survival[I, Random(MaxState) + 1] := Random(SurvivalRandomLimit)
  end;
  InitRuleGrids
end;

procedure TInputDialog.OKButtonClick(Sender: TObject);
begin
  ModalResult := mrOK
end;

procedure TInputDialog.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel
end;

procedure SaveRuleString(S: String);
begin
  RuleString := S
end;

procedure InitBirths;
var
  I, J: Integer;
begin
  for I := 1 to MaxState do
    for J := 1 to MaxState do
      Birth[I, J] := 0
end;

procedure InitSurvivals;
var
  I, J: Integer;
begin
  for I := 1 to MaxState do
    for J := 1 to MaxState do
      Survival[I, J] := 0
end;

procedure InitVars;
begin
  BirthTestOperators := OR_GE;
  SurvivalTestOperators := OR_GE;
  InitBirths;
  InitSurvivals
end;


function SaveRuleSet(S: String; K: Integer): Integer;
var
  T: String;
  I, J: Integer;
begin
  T := S[K];
  I := StrToInt(Copy(S, K + 1, 1));
  K := K + 3;
  J := 1;
  while (K <= Length(S)) and (S[K] <> ',') do
  begin
    if T = 'B' then
      Birth[I, J] := StrToInt(S[K])
    else
      Survival[I, J] := StrToInt(S[K]);
    Inc(J);
    Inc(K)
  end;
  Result := K + 1
end;

procedure InitRuleFromString(S: String);
var
  I: Integer;
begin
  if S = '' then
    InitVars
  else
  begin
    BirthTestOperators := TestOperators(StrToInt(S[3]));
    SurvivalTestOperators := TestOperators(StrToInt(S[7]));
    I := 9;
    while I < Length(S) do
      I := SaveRuleSet(S, I)
  end
end;

function GetRuleAsString: String;
var
  S: String;
  I, J: Integer;
begin
  S := 'TB' + IntToStr(Ord(BirthTestOperators)) + ',';
  S := S + 'TS' + IntToStr(Ord(SurvivalTestOperators)) + ',';
  for I := 1 to MaxState do
  begin
    S := S + 'B' + IntToStr(I) + ';';
    for J := 1 to MaxState do
      S := S + IntToStr(Birth[I, J]);
    S := S + ','
  end;
  for I := 1 to MaxState do
  begin
    S := S + 'S' + IntToStr(I) + ';';
    for J := 1 to MaxState do
      S := S + IntToStr(Survival[I, J]);
    if I <> MaxState then
      S := S + ','
  end;
  Result := S
end;

end.
